@@ -23,6 +23,7 @@
 #include <linux/nexthop.h>
 
 #include "libnetlink.h"
+#include "json_print.h"
 #include "utils.h"
 
 #ifndef __aligned
@@ -111,6 +112,10 @@ int nl_dump_ext_ack(const struct nlmsghdr *nlh, nl_ext_ack_fn_t errfn)
 			err_nlh = &err->msg;
 	}
 
+	if (tb[NLMSGERR_ATTR_MISS_TYPE])
+		fprintf(stderr, "Missing required attribute type %u\n",
+			mnl_attr_get_u32(tb[NLMSGERR_ATTR_MISS_TYPE]));
+
 	if (errfn)
 		return errfn(msg, off, err_nlh);
 
@@ -727,13 +732,15 @@ int rtnl_dump_request_n(struct rtnl_handle *rth, struct nlmsghdr *n)
 static int rtnl_dump_done(struct nlmsghdr *h,
 			  const struct rtnl_dump_filter_arg *a)
 {
-	int len = *(int *)NLMSG_DATA(h);
+	int len;
 
 	if (h->nlmsg_len < NLMSG_LENGTH(sizeof(int))) {
 		fprintf(stderr, "DONE truncated\n");
 		return -1;
 	}
 
+	len = *(int *)NLMSG_DATA(h);
+
 	if (len < 0) {
 		errno = -len;
 
@@ -1140,7 +1147,7 @@ static int __rtnl_talk(struct rtnl_handle *rtnl, struct nlmsghdr *n,
 	return __rtnl_talk_iov(rtnl, &iov, 1, answer, show_rtnl_err, errfn);
 }
 
-int rtnl_echo_talk(struct rtnl_handle *rtnl, struct nlmsghdr *n, int json,
+int rtnl_echo_talk(struct rtnl_handle *rtnl, struct nlmsghdr *n, int use_json,
 		   int (*print_info)(struct nlmsghdr *n, void *arg))
 {
 	struct nlmsghdr *answer;
@@ -1152,7 +1159,7 @@ int rtnl_echo_talk(struct rtnl_handle *rtnl, struct nlmsghdr *n, int json,
 	if (ret)
 		return ret;
 
-	new_json_obj(json);
+	new_json_obj(use_json);
 	open_json_object(NULL);
 	print_info(answer, stdout);
 	close_json_object();
@@ -1168,12 +1175,6 @@ int rtnl_talk(struct rtnl_handle *rtnl, struct nlmsghdr *n,
 	return __rtnl_talk(rtnl, n, answer, true, NULL);
 }
 
-int rtnl_talk_iov(struct rtnl_handle *rtnl, struct iovec *iovec, size_t iovlen,
-		  struct nlmsghdr **answer)
-{
-	return __rtnl_talk_iov(rtnl, iovec, iovlen, answer, true, NULL);
-}
-
 int rtnl_talk_suppress_rtnl_errmsg(struct rtnl_handle *rtnl, struct nlmsghdr *n,
 				   struct nlmsghdr **answer)
 {
@@ -1561,78 +1562,6 @@ int __parse_rtattr_nested_compat(struct rtattr *tb[], int max,
 	return 0;
 }
 
-static const char *get_nla_type_str(unsigned int attr)
-{
-	switch (attr) {
-#define C(x) case NL_ATTR_TYPE_ ## x: return #x
-	C(U8);
-	C(U16);
-	C(U32);
-	C(U64);
-	C(STRING);
-	C(FLAG);
-	C(NESTED);
-	C(NESTED_ARRAY);
-	C(NUL_STRING);
-	C(BINARY);
-	C(S8);
-	C(S16);
-	C(S32);
-	C(S64);
-	C(BITFIELD32);
-	default:
-		return "unknown";
-	}
-}
-
-void nl_print_policy(const struct rtattr *attr, FILE *fp)
-{
-	const struct rtattr *pos;
-
-	rtattr_for_each_nested(pos, attr) {
-		const struct rtattr *attr;
-
-		fprintf(fp, " policy[%u]:", pos->rta_type & ~NLA_F_NESTED);
-
-		rtattr_for_each_nested(attr, pos) {
-			struct rtattr *tp[NL_POLICY_TYPE_ATTR_MAX + 1];
-
-			parse_rtattr_nested(tp, ARRAY_SIZE(tp) - 1, attr);
-
-			if (tp[NL_POLICY_TYPE_ATTR_TYPE])
-				fprintf(fp, "attr[%u]: type=%s",
-					attr->rta_type & ~NLA_F_NESTED,
-					get_nla_type_str(rta_getattr_u32(tp[NL_POLICY_TYPE_ATTR_TYPE])));
-
-			if (tp[NL_POLICY_TYPE_ATTR_POLICY_IDX])
-				fprintf(fp, " policy:%u",
-					rta_getattr_u32(tp[NL_POLICY_TYPE_ATTR_POLICY_IDX]));
-
-			if (tp[NL_POLICY_TYPE_ATTR_POLICY_MAXTYPE])
-				fprintf(fp, " maxattr:%u",
-					rta_getattr_u32(tp[NL_POLICY_TYPE_ATTR_POLICY_MAXTYPE]));
-
-			if (tp[NL_POLICY_TYPE_ATTR_MIN_VALUE_S] && tp[NL_POLICY_TYPE_ATTR_MAX_VALUE_S])
-				fprintf(fp, " range:[%lld,%lld]",
-					(signed long long)rta_getattr_u64(tp[NL_POLICY_TYPE_ATTR_MIN_VALUE_S]),
-					(signed long long)rta_getattr_u64(tp[NL_POLICY_TYPE_ATTR_MAX_VALUE_S]));
-
-			if (tp[NL_POLICY_TYPE_ATTR_MIN_VALUE_U] && tp[NL_POLICY_TYPE_ATTR_MAX_VALUE_U])
-				fprintf(fp, " range:[%llu,%llu]",
-					(unsigned long long)rta_getattr_u64(tp[NL_POLICY_TYPE_ATTR_MIN_VALUE_U]),
-					(unsigned long long)rta_getattr_u64(tp[NL_POLICY_TYPE_ATTR_MAX_VALUE_U]));
-
-			if (tp[NL_POLICY_TYPE_ATTR_MIN_LENGTH])
-				fprintf(fp, " min len:%u",
-					rta_getattr_u32(tp[NL_POLICY_TYPE_ATTR_MIN_LENGTH]));
-
-			if (tp[NL_POLICY_TYPE_ATTR_MAX_LENGTH])
-				fprintf(fp, " max len:%u",
-					rta_getattr_u32(tp[NL_POLICY_TYPE_ATTR_MAX_LENGTH]));
-		}
-	}
-}
-
 int rtnl_tunneldump_req(struct rtnl_handle *rth, int family, int ifindex,
 			__u8 flags)
 {
